PRO outmes, message, infow = infow
IF NOT keyword_set(infow) THEN print, message $
  ELSE widget_control, infow, set_value = message, /append
END


function crea_psf, siz, fwhmx, fwhmy

result=fltarr(siz,siz)

xar=findgen(siz)#replicate(1.,siz) ; array of x values
yar=replicate(1.,siz)#findgen(siz) ; same for y
fwhmx = fwhmx/sqrt(8.0*alog(2)) ; adjust sigma to fwhm
fwhmy = fwhmy/sqrt(8.0*alog(2))
zx = (xar-siz/2)/fwhmx
zy = (yar-siz/2)/fwhmy
ez = exp(-zx^2/2)*exp(-zy^2/2) 
result=ez/total(ez)

return, result
end 



pro vs_mapmake, comfile,  outfile=outfile, lambdas = lambdas, infow = infow
COMMON vs_block
IF NOT vs_debug THEN on_error, 2


outfile = 'errorerrorerror';in case something goes wrong...

IF n_params() LT 1 THEN BEGIN 
    vs_info, 'No input file given!', 4
    return
ENDIF 


;catch, ErrorNumber              ; Catch any unforeseen error
;IF ErrorNumber NE 0 THEN BEGIN
;    catch, /cancel
;    IF N_Elements(ProgID) EQ 3 THEN Dummy = vs_progbar(ProgID, -1) ; destroy the progress bar
;    vs_info, 'Error in (with) commandfile: '+comfile, 4 ; announce error
;    return
;ENDIF 

ProgID = vs_progbar([0, 0, 0], 0, action='Reading input file...')
num_commands = 9
command = {com, nam:'', num_args:0, required:0, syntax:''}
commands = replicate({com}, num_commands)
commands[0] = {com, nam:'spike', num_args:3, required:0, syntax:'SPIKE x y inten'}
commands[1] = {com, nam:'ellipse',  num_args:8, required:0, syntax:'ELLIPSE x y inten a_out ratio angle a_in fwhm'}
commands[2] = {com, nam:'rectang',  num_args:8, required:0, syntax:'RECTANG x y inten a_out ratio angle a_in fwhm'}
commands[3] = {com, nam:'imsize',  num_args:1, required:1, syntax:'IMSIZE n'}
commands[4] = {com, nam:'pixsize',  num_args:1, required:1, syntax:'PIXSIZE xx.xxx'}
commands[5] = {com, nam:'lambda',  num_args:1, required:0, syntax:'LAMBDA xx.xxx'}
commands[6] = {com, nam:'filename',  num_args:1, required:1, syntax:'FILENAME name'}
commands[7] = {com, nam:'gauss', num_args:6, required:0, syntax:'GAUSS x y inten fwhm ratio angle'}
commands[8] = {com, nam:'tag', num_args:1, required:0, syntax:'TAG tagline'}
lambdas = fltarr(5)
tags = strarr(5)
num_errors = 0

;;open file end check for errors
get_lun,lun
openr,lun, comfile, ERROR = err
IF err NE 0 THEN BEGIN
    close, lun
    vs_info, 'Error opening file: '+comfile, 4
    Dummy = vs_progbar(ProgID, -1)
   return 
ENDIF
    line = '' & READF, lun, line
    
;; command block of file should start with '--'
    while (strpos("--", strmid(line, 0, 1)) eq -1) or strlen(line) eq 0 do begin
        readf, lun, line
    endwhile

;; now read the commands
    num_lines = 0
    num_required = total(commands.required)
    num_reqfound = 0

    WHILE  NOT  EOF(lun) DO  BEGIN 
        line = '' & READF, lun, line
        c_pos = strpos(line, ';')
        IF total(c_pos) NE -1 THEN line = strmid(line, 0, c_pos[0])
        line = strtrim(line, 2)
        
        IF strlen(line) NE 0 THEN BEGIN 
            IF vs_debug THEN print, line
            num_lines = num_lines+1
            IF num_lines EQ 1 THEN BEGIN
                c = strarr(1)
                c(0) = line
            ENDIF ELSE BEGIN
                cn = strarr(num_lines)
                cn(0:num_lines-2) = c
                cn(num_lines-1) = line
                c = cn
            ENDELSE
        ENDIF
    ENDWHILE
    close, lun


IF num_lines LT num_required THEN BEGIN
    vs_infi, 'Not even enough lines to provide required commands in file '+comfile, 4
    num_errors = num_errors+1
ENDIF

;; now the parsing for the commands

FOR j = 0, num_required-1 DO BEGIN    ;these must! come first
    com_fields = str_sep(strcompress(c[j]), ' ')
    c_ind = where(commands.nam EQ strlowcase(com_fields[0]))
    IF total(c_ind) EQ -1 THEN BEGIN
        vs_info, 'Unknown command: '+com_fields[0], 4
        num_errors = num_errors+1
    ENDIF ELSE begin
        IF commands[c_ind].required EQ 0 THEN BEGIN
            vs_info, 'Found non-required command: '+com_fields[0], 4
            num_errors = num_errors+1
        ENDIF
        IF n_elements(com_fields) NE commands[c_ind].num_args +1 THEN BEGIN
            vs_info, 'Syntax error, usage: '+commands[c_ind].syntax, 4
            num_errors = num_errors+1
        ENDIF
        IF num_errors EQ 0 THEN BEGIN
            CASE total(c_ind) OF 
                3: imsize = fix(com_fields[1])
                4: pixsize = float(com_fields[1])
                6: outfile = com_fields[1]
            ENDCASE
            num_reqfound = num_reqfound+1 ; one read without error
        ENDIF
    ENDELSE
ENDFOR

IF num_reqfound NE num_required THEN BEGIN
    vs_info, 'Missing one or more required commands!', 4
    Dummy = vs_progbar(ProgID, -1)
    return
ENDIF 

; now start making the array and read the rest
res_array = fltarr(imsize, imsize, 5)
n_image = 0
lambdas[0] = 1e-5;initialize default of 10 microns for first image

FOR j = num_required, num_lines-1 DO BEGIN
    com_fields = str_sep(strlowcase(strcompress(c[j])), ' ')
    c_ind = where(commands.nam EQ com_fields[0])
    IF total(c_ind) EQ -1 THEN BEGIN
        vs_info, 'Unknown command: '+com_fields[0], 4
        num_errors = num_errors+1
    ENDIF ELSE BEGIN
        IF n_elements(com_fields) NE commands[c_ind].num_args +1 THEN BEGIN
            vs_info, 'Syntax error, usage: '+commands[c_ind].syntax, 4
            num_errors = num_errors+1
        ENDIF
        IF num_errors EQ 0 THEN BEGIN
            add_array = res_array[*, *, n_image]*0.0
            CASE total(c_ind) OF 
                0:BEGIN
                    Dummy = vs_progbar(ProgID, $
                                       j/float(num_lines)*90., $
                                       action='Making spike in frame '+$
                                       string(n_image, format='(i1.1)')+$
                                       '...')
                    x = fix(com_fields[1])
                    y = fix(com_fields[2])
                    inten = float(com_fields[3])
                    add_array[x, y] = inten
                END
                1: BEGIN
                    Dummy = vs_progbar(ProgID, $
                                       j/float(num_lines)*90., $
                                       action='Making ellipse in frame '+$
                                       string(n_image, format='(i1.1)')+$
                                       '...')
                    x = fix(com_fields[1])
                    y = fix(com_fields[2])
                    inten = float(com_fields[3])
                    a = fix(com_fields[4])
                    ratio = float(com_fields[5])
                    angle = float(com_fields[6])
                    a_in = float(com_fields[7])
                    g = float(com_fields[8])
                    dist_ellipse, el_array, imsize, x, y, ratio, angle
                    ones = where(el_array LE a, n_ones)
                    IF n_ones GT 0 THEN  BEGIN
                        add_array[ones] = 1.
                        IF a_in NE 0 THEN $
                          add_array [where(el_array LE a*a_in)]=0.
                    ENDIF
                    IF total(add_array) NE 0 THEN begin
                        add_array = add_array/total(add_array)*inten
                        IF g NE 0 THEN BEGIN
                            psf = crea_psf(imsize, g, g)
                            add_array = convolve(add_array, psf)
                        ENDIF                          
                    ENDIF ELSE vs_info, 'Warning,  no flux in ellipse...', 3
                END
                2: BEGIN
                    Dummy = vs_progbar(ProgID, $
                                       j/float(num_lines)*90., $
                                       action='Making rectangle in frame '+$
                                       string(n_image, format='(i1.1)')+$
                                       '...')
                    x = fix(com_fields[1])
                    y = fix(com_fields[2])
                    inten = float(com_fields[3])
                    a = fix(com_fields[4])
                    ratio = float(com_fields[5])
                    angle = float(com_fields[6])
                    a_in = float(com_fields[7])
                    g = float(com_fields[8])
                    add_array[x-a/ratio:x+a/ratio, y-a:y+a] = 1.
                    
                    IF a_in NE 0 THEN $
                      add_array[x-a*a_in/ratio:x+a*a_in/ratio, y-a*a_in:y+a*a_in]=0.
                    IF angle NE 0 THEN $
                      add_array=rot(add_array, -angle, 1.0, x, y)
                    IF total(add_array) NE 0 THEN BEGIN 
                        add_array = add_array/total(add_array)*inten
                        IF g NE 0 THEN BEGIN
                            psf = crea_psf(imsize, g, g)
                            add_array = convolve(add_array, psf)
                        ENDIF        
                    ENDIF ELSE vs_info, 'No flux in rectangle...',3
                END
                5:BEGIN
                    Dummy = vs_progbar(ProgID, $
                                       j/float(num_lines)*90., $
                                       action='Starting new frame with wavelength '+$
                                       com_fields[1])
                    IF j NE num_required THEN n_image = n_image+1
                    lambdas[n_image] = float(com_fields[1])
                END
                7:BEGIN
                    Dummy = vs_progbar(ProgID, $
                                       j/float(num_lines)*90., $
                                       action='Making Gaussian in frame '+$
                                       string(n_image, format='(i1.1)')+$
                                       '...')
                    x = fix(com_fields[1])
                    y = fix(com_fields[2])
                    inten = float(com_fields[3])
                    fx = float(com_fields[4])
                    ratio = float(com_fields[5])
                    angle = float(com_fields[6])
                    add_array = crea_psf(imsize, fx, fx*ratio)
                    IF angle NE 0 THEN add_array = rot(add_array, -angle)
                    add_array = shift(add_array, x-imsize/2., y-imsize/2.)
                    add_array = add_array/total(add_array)*inten
                END
                8:BEGIN
                    Dummy = vs_progbar(ProgID, $
                                       j/float(num_lines)*90., $
                                       action='Adding comment line in current frame: '+$
                                       com_fields[1])
                    tags[n_image] = com_fields[1]
                END 
            ENDCASE 
            
            res_array[*, *, n_image] = res_array[*, *, n_image]+add_array
        ENDIF
    ENDELSE
ENDFOR

IF num_errors EQ 0 THEN BEGIN 
    res_array = reform(res_array[*, *, 0:n_image])
    sxhmake, res_array, 1, imhead
    sxaddpar, imhead, 'PIXSIZE', pixsize
    FOR i = 1, 5 DO BEGIN
        sxaddpar, imhead, 'LAMBDA'+string(i, format='(i1.1)'), lambdas[i-1]
        sxaddpar, imhead, 'TAG'+string(i, format='(i1.1)'), tags[i-1]
    ENDFOR 
    sxaddpar, imhead, 'CALUNIT', 'mJY/PIX'
    writefits, outfile, res_array, imhead
    Dummy = vs_progbar(ProgID, -1, action='Fits-file creation done!')
ENDIF ELSE BEGIN
    vs_info, 'There have been errors, no output produced!',4
    Dummy = vs_progbar(ProgID, -1)
ENDELSE 
END
